@@ -28,7 +28,7 @@ from openid.yadis.constants import YADIS_CONTENT_TYPE
 
 from openid_provider import conf
 from openid_provider.utils import add_sreg_data, add_ax_data, get_store, \
-    trust_root_validation, get_trust_session_key
+    trust_root_validation, get_trust_session_key, prep_response
 from openid_provider.models import TrustedRoot
 
 logger = logging.getLogger(__name__)
@@ -41,8 +41,7 @@ def openid_server(request):
     """
     logger.debug('server request %s: %s',
                  request.method, request.POST or request.GET)
-    server = Server(get_store(request),
-        op_endpoint=request.build_absolute_uri(reverse('openid-provider-root')))
+    server = openid_get_server(request)
 
     if not request.is_secure():
         # if request is not secure allow only encrypted association sessions
@@ -55,7 +54,7 @@ def openid_server(request):
     querydict = dict(request.REQUEST.items())
     orequest = server.decodeRequest(querydict)
     if not orequest:
-        orequest = request.session.get('OPENID_REQUEST', None)
+        orequest = server.decodeRequest(request.session.get('OPENID_REQUEST', None))
         if orequest:
             # remove session stored data:
             del request.session['OPENID_REQUEST']
@@ -100,7 +99,7 @@ def openid_server(request):
             logger.debug('checkid_immediate mode not supported')
             raise Exception('checkid_immediate mode not supported')
         else:
-            request.session['OPENID_REQUEST'] = orequest
+            request.session['OPENID_REQUEST'] = orequest.message.toPostArgs()
             request.session['OPENID_TRUSTROOT_VALID'] = trust_root_valid
             logger.debug('redirecting to decide page')
             return HttpResponseRedirect(reverse('openid-provider-decide'))
@@ -110,20 +109,8 @@ def openid_server(request):
         add_sreg_data(request, orequest, oresponse)
         if conf.AX_EXTENSION:
             add_ax_data(request, orequest, oresponse)
-    # Convert a webresponse from the OpenID library in to a Django HttpResponse
-    webresponse = server.encodeResponse(oresponse)
-    if webresponse.code == 200 and orequest.mode in BROWSER_REQUEST_MODES:
-        response = render_to_response('openid_provider/response.html', {
-            'body': webresponse.body,
-        }, context_instance=RequestContext(request))
-        logger.debug('rendering browser response')
-    else:
-        response = HttpResponse(webresponse.body)
-        response.status_code = webresponse.code
-        for key, value in webresponse.headers.items():
-            response[key] = value
-        logger.debug('rendering raw response')
-    return response
+
+    return prep_response(request, orequest, oresponse, server)
 
 def openid_xrds(request, identity=False, id=None):
     if identity:
@@ -146,7 +133,8 @@ def openid_decide(request):
     # If user is logged in, ask if they want to trust this trust_root
     # If they are NOT logged in, show the landing page
     """
-    orequest = request.session.get('OPENID_REQUEST')
+    server = openid_get_server(request)
+    orequest = server.decodeRequest(request.session.get('OPENID_REQUEST'))
     trust_root_valid = request.session.get('OPENID_TRUSTROOT_VALID')
 
     if not request.user.is_authenticated():
@@ -158,11 +146,16 @@ def openid_decide(request):
             request, "You are signed in but you don't have OpenID here!")
 
     if request.method == 'POST' and request.POST.get('decide_page', False):
-        TrustedRoot.objects.get_or_create(
-            openid=openid, trust_root=orequest.trust_root)
-        if not conf.FAILED_DISCOVERY_AS_VALID:
-            request.session[get_trust_session_key(orequest)] = True
-        return HttpResponseRedirect(reverse('openid-provider-root'))
+        if request.POST.get('allow', False):
+            TrustedRoot.objects.get_or_create(
+                openid=openid, trust_root=orequest.trust_root)
+            if not conf.FAILED_DISCOVERY_AS_VALID:
+                request.session[get_trust_session_key(orequest)] = True
+            return HttpResponseRedirect(reverse('openid-provider-root'))
+
+        oresponse = orequest.answer(False)
+        logger.debug('orequest.answer(False)')
+        return prep_response(request, orequest, oresponse)
 
     return render_to_response('openid_provider/decide.html', {
         'title': _('Trust this site?'),
@@ -204,7 +197,7 @@ def landing_page(request, orequest, login_url=None,
     but is not authenticated with the site. For idproxy.net, a message telling
     them to log in manually is displayed.
     """
-    request.session['OPENID_REQUEST'] = orequest
+    request.session['OPENID_REQUEST'] = orequest.message.toPostArgs()
     if not login_url:
         login_url = settings.LOGIN_URL
     path = request.get_full_path()
@@ -253,3 +246,10 @@ def openid_get_identity(request, identity_url):
         if request.user.openid_set.count() > 0:
             return request.user.openid_set.all()[0]
     return None
+
+
+def openid_get_server(request):
+    return Server(
+        get_store(request),
+        op_endpoint=request.build_absolute_uri(
+            reverse('openid-provider-root')))
